"""gerridae log"""
import logging


def get_logger(name='gerridae'):
    logger_format = "[%(asctime)s] %(levelname)s [%(name)s] %(message)s"

    logging.basicConfig(
        format=logger_format,
        level=logging.INFO,
        datefmt='%Y:%m:%d %H:%M:%S'
    )

    return logging.getLogger(name)



from gerridae.field import BaseField
import requests
from lxml import etree


class ItemMeta(type):

    def __new__(mcs, name, bases, attrs):
        __fields = {field_name: attrs.pop(field_name) for field_name, obj in list(attrs.items()) if
                    isinstance(obj, BaseField)}
        attrs['__fields'] = __fields

        return type.__new__(mcs, name, bases, attrs)


class Item(metaclass=ItemMeta):

    def __init__(self):
        self.result = {}

    def __repr__(self):
        return f'<{self.__class__.__name__}> {self.result}'

    @staticmethod
    def _get_html(html, url, **kwargs):
        """获取 etree.HTML obj"""
        if html or url:
            if url:
                res = requests.get(url, **kwargs)
                if res.apparent_encoding != res.encoding:
                    res.encoding = 'utf-8'
                html = res.text
                return etree.HTML(html)
            return etree.HTML(html)
        raise ValueError('html or url is expected')

    @classmethod
    def _parser_html(cls, *, html_etree):
        if html_etree is None:
            return ValueError('html_etree is expected')
        item_ins = cls()

        fields = getattr(item_ins, '__fields', {})
        for field_name, field_value in fields.items():
            clean_method = getattr(item_ins, f'clean_{field_name}', None)
            value = field_value.extract(html_etree)

            # check the many to get the current
            many = getattr(field_value, 'many', False)
            if not many:
                value = value if len(value) == 0 else value[0]

            if clean_method and callable(clean_method):
                assert value, f'{field_name} value is {value}'
                value = clean_method(value)

            # default value for field
            if not value:
                default_value = getattr(field_value, 'default', False)
                value = default_value

            # setattr for Item class
            setattr(item_ins, field_name, value)
            item_ins.result[field_name] = value
        return item_ins

    @classmethod
    def get_item(cls, html=None, url=None):
        if html or url:
            html_etree = cls._get_html(html, url)
            return cls._parser_html(html_etree=html_etree)
        raise ValueError('html or url is excepted')



from setuptools import setup, find_packages

__version__ = '0.0.8'

setup(
    name='gerridae',
    version=__version__,
    author='buglan',
    packages=find_packages(),
    description='a crawl framework just for fun',
    author_email='dev.buglan@gmail.com',
    python_requires='>=3.6',
    install_requires=['cssselect', 'lxml', 'requests'],
)



import requests
from datetime import datetime
from gerridae.log import logger


class Spider:
    start_urls = []

    def _start(self, url):
        pass

    def parse(self, response):
        raise NotImplementedError

    @classmethod
    def start(cls):
        start_time = datetime.now()
        spider_ins = cls()
        if isinstance(spider_ins.start_urls, list):
            data = []
            for url in cls.start_urls:
                result = spider_ins.parse(requests.get(url, headers={
                    'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.100 Safari/537.36'}))
                data.append(result)
        elif isinstance(spider_ins.start_urls, str):
            data = spider_ins.parse(requests.get(spider_ins.start_urls, headers={
                'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.100 Safari/537.36'}))
        else:
            raise ValueError('start_urls must be str or list')

        logger.info(f'total time is {datetime.now() - start_time}')
        return data



class BaseField:
    """base field"""

    def __init__(self, many=False, default=None):
        self.default = default
        self.many = many

    def extract(self, *args, **kwargs):
        raise NotImplementedError('extract is not implemented.')


class _LXMLElementField(BaseField):
    """usage TextField Attribute """

    def __init__(self, css_select=None, many=False, default=None):
        super(_LXMLElementField, self).__init__(many, default)
        self.css_select = css_select
        self.many = many

    def _get_elements(self, *, html_etree):
        if self.css_select:
            elements = html_etree.cssselect(self.css_select)
        else:
            raise ValueError(f'{self.__class__.__name__} field css_select is expected')
        return elements

    def _parser_element(self, element):
        raise NotImplementedError

    def extract(self, html_etree):
        elements = self._get_elements(html_etree=html_etree)
        if elements:
            result = [self._parser_element(element) for element in elements]
        else:
            result = []
        return result


class TextField(_LXMLElementField):
    """text filed"""

    def __init__(self, css_select=None, many=False, default=None):
        super(TextField, self).__init__(css_select, many, default)

    def _parser_element(self, element):
        strings = [node.strip() for node in element.itertext()]
        string = ''.join(strings)
        return string if string else self.default


class AttrField(_LXMLElementField):
    """attribute field"""

    def __init__(self, css_select=None, attr=None, many=False, default=None):
        super(AttrField, self).__init__(css_select, many, default)
        assert attr, 'excepted attr'
        self.attr = attr

    def _parser_element(self, element):
        return element.get(self.attr, self.default)



"""
test the integer field
"""

from gerridae import Item, TextField, Spider


class BaiduItem(Item):
    name = TextField(css_select='#su')


class BaiduSpider(Spider):
    start_urls = ['https://www.baidu.com']

    def parse(self, response):
        result = BaiduItem.get_item(html=response.text)
        assert result


def test_spider():
    BaiduSpider.start()


if __name__ == '__main__':
    test_spider()
